import java.applet.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.awt.event.*; // Event handling class are defined here.
import javax.swing.*;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.TargetDataLine;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.AudioFileFormat;


public class RecordApplet extends JApplet {

    public SimpleAudioRecorder  _recorder;

    public String _save_path;
    public String _file_id;
    public String _status;
    public String _file;

    private AudioClip sound;

    private static final int	EXTERNAL_BUFFER_SIZE = 128000;

    public void init() {
            // This method is called by the system before the applet
            // appears.  It is used here to create the button and add
            // it to the "content pane" of the JApplet.  An anonymous
            // class is used to create an ActionListener for the button.

        String s;

        s = getParameter("save_path");
        if (s != null)
            this._save_path = s;
        else {
            set_status("Error: Save path not set.");
			return;
        }

        s = getParameter("file_id");
        if (s != null)
            this._file_id = s;
        else {
            set_status("Error: File id not set.");
			return;
        }

        this._file = this._save_path + this._file_id + "_prompt.wav";

        JButton record_btn = new JButton("Record");
        JButton stop_btn = new JButton("Stop");
        JButton play_btn = new JButton("Play");

        record_btn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) { do_record(); }
        });
        stop_btn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) { do_stop(); }
        });
        play_btn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) { do_play(); }
        });

        /*
         bttn.addActionListener( new ActionListener() {
                  // The "action listener" for the button is defined
                  // by this nested anonymous class.
               public void actionPerformed(ActionEvent evt) {
                     // This method is called to respond when the user
                     // presses the button.  It displays a message in 
                     // a dialog box, along with an "OK" button which
                     // the user can click to dismiss the dialog box.
                  String title = "Greetings";  // Shown in box's title bar.
                  String message = "Another hello from Swing.";
                  JOptionPane.showMessageDialog(null, message, title,
                                        JOptionPane.INFORMATION_MESSAGE);
               } // end actionPerformed()
            });
            */

        //Lay out the GUI.
        GridBagLayout layout = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();

        Container contentPane = getContentPane();
        contentPane.setLayout(layout);
        contentPane.add(record_btn);
        contentPane.add(stop_btn);
        contentPane.add(play_btn);

        set_status("Recorder ready.");
    }

    public void paint(Graphics g) {

        super.paint(g);

        g.drawString(this._status, 20, 60);
    } 

    public void do_record() {

		File outputFile = new File(this._file);

		/* For simplicity, the audio data format used for recording
		   is hardcoded here. We use PCM 44.1 kHz, 16 bit signed,
		   stereo.
		*/
		AudioFormat	audioFormat = new AudioFormat(
			AudioFormat.Encoding.PCM_SIGNED,
			44100.0F, 16, 2, 4, 44100.0F, false);

		/* Now, we are trying to get a TargetDataLine. The
		   TargetDataLine is used later to read audio data from it.
		   If requesting the line was successful, we are opening
		   it (important!).
		*/
		DataLine.Info	info = new DataLine.Info(TargetDataLine.class, audioFormat);
		TargetDataLine	targetDataLine = null;
		try
		{
			targetDataLine = (TargetDataLine) AudioSystem.getLine(info);
			targetDataLine.open(audioFormat);
		}
		catch (LineUnavailableException e)
		{
            set_status("unable to get a recording line");
			e.printStackTrace();
			System.exit(1);
		}

		/* Again for simplicity, we've hardcoded the audio file
		   type, too.
		*/
		AudioFileFormat.Type	targetType = AudioFileFormat.Type.WAVE;

		/* Now, we are creating an SimpleAudioRecorder object. It
		   contains the logic of starting and stopping the
		   recording, reading audio data from the TargetDataLine
		   and writing the data to a file.
		*/
		this._recorder = new SimpleAudioRecorder(
			targetDataLine,
			targetType,
			outputFile);

		/* We are waiting for the user to press ENTER to
		   start the recording. (You might find it
		   inconvenient if recording starts immediately.)
		out("Press ENTER to start the recording.");
		try
		{
			System.in.read();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		*/
		/* Here, the recording is actually started.
		 */
		this._recorder.start();
		set_status("Recording...");
    }
    public void do_stop() {
		this._recorder.stopRecording();
		set_status("Recording stopped.");
    }
    public void do_play() {

        do_stop();

		set_status("Playing...");

        /*
		  Now, that we're shure there is an argument, we
		  take it as the filename of the soundfile
		  we want to play.
		*/
		File	soundFile = new File(this._file);
	
		/*
		  We have to read in the sound file.
		*/
		AudioInputStream	audioInputStream = null;
		try
		{
			audioInputStream = AudioSystem.getAudioInputStream(soundFile);
		}
		catch (Exception e)
		{
			/*
			  In case of an exception, we dump the exception
			  including the stack trace to the console output.
			  Then, we exit the program.
			*/
            set_status("Error: cannot get AudioInputStream");
			e.printStackTrace();
            return;
		}

		/*
		  From the AudioInputStream, i.e. from the sound file,
		  we fetch information about the format of the
		  audio data.
		  These information include the sampling frequency,
		  the number of
		  channels and the size of the samples.
		  These information
		  are needed to ask Java Sound for a suitable output line
		  for this audio file.
		*/
		AudioFormat	audioFormat = audioInputStream.getFormat();

		/*
		  Asking for a line is a rather tricky thing.
		  We have to construct an Info object that specifies
		  the desired properties for the line.
		  First, we have to say which kind of line we want. The
		  possibilities are: SourceDataLine (for playback), Clip
		  (for repeated playback)	and TargetDataLine (for
		  recording).
		  Here, we want to do normal playback, so we ask for
		  a SourceDataLine.
		  Then, we have to pass an AudioFormat object, so that
		  the Line knows which format the data passed to it
		  will have.
		  Furthermore, we can give Java Sound a hint about how
		  big the internal buffer for the line should be. This
		  isn't used here, signaling that we
		  don't care about the exact size. Java Sound will use
		  some default value for the buffer size.
		*/
		SourceDataLine	line = null;
		DataLine.Info	info = new DataLine.Info(SourceDataLine.class,
												 audioFormat);
		try
		{
			line = (SourceDataLine) AudioSystem.getLine(info);

			/*
			  The line is there, but it is not yet ready to
			  receive audio data. We have to open the line.
			*/
			line.open(audioFormat);
		}
		catch (LineUnavailableException e)
		{
			e.printStackTrace();
            set_status("Error: DataLine");
			System.exit(1);
		}
		catch (Exception e)
		{
			e.printStackTrace();
            set_status("Error: DataLine");
			System.exit(1);
		}

		/*
		  Still not enough. The line now can receive data,
		  but will not pass them on to the audio output device
		  (which means to your sound card). This has to be
		  activated.
		*/
		line.start();

		/*
		  Ok, finally the line is prepared. Now comes the real
		  job: we have to write data to the line. We do this
		  in a loop. First, we read data from the
		  AudioInputStream to a buffer. Then, we write from
		  this buffer to the Line. This is done until the end
		  of the file is reached, which is detected by a
		  return value of -1 from the read method of the
		  AudioInputStream.
		*/
		int	nBytesRead = 0;
		byte[]	abData = new byte[EXTERNAL_BUFFER_SIZE];
		while (nBytesRead != -1)
		{
			try
			{
				nBytesRead = audioInputStream.read(abData, 0, abData.length);
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
			if (nBytesRead >= 0)
			{
				int	nBytesWritten = line.write(abData, 0, nBytesRead);
			}
		}

		/*
		  Wait until all data are played.
		  This is only necessary because of the bug noted below.
		  (If we do not wait, we would interrupt the playback by
		  prematurely closing the line and exiting the VM.)
		 
		  Thanks to Margie Fitch for bringing me on the right
		  path to this solution.
		*/
		line.drain();

		/*
		  All data are played. We can close the shop.
		*/
		line.close();

		/*
		  There is a bug in the jdk1.3/1.4.
		  It prevents correct termination of the VM.
		  So we have to exit ourselves.
		System.exit(0);
		*/

		set_status("Recorder ready.");
    }

    private void set_status(String status) {

        this._status = status;

        System.out.println(this._status);

        repaint();
    }
}
